
type 'a t = ('a,bool) Hashtbl.t

let create () = Hashtbl.create 100

let empty = create

let add s a = Hashtbl.replace s a true
let (++) s x = add s x ; s

let make l = 
  let s = create () in
    List.iter (add s) l;
    s

let remove s a = Hashtbl.replace s a false

let mem s a = 
  try Hashtbl.find s a 
  with Not_found -> false

let iter s f = Hashtbl.iter (fun a x -> if x then f a) s

let fold f s init =
  Hashtbl.fold (fun k d x -> if d then f k x else x) s init

let to_string de s = 
  "{"^ fold (fun x a -> a ^ de x ^";") s "" ^ "}"



let remove s x = Hashtbl.replace s x false
let (--) s x = remove s x; s

let intersect s t = 
  let r = empty () in
    iter s (fun x -> if mem t x then add r x);
    r
  
let ( ** ) s t = intersect s t

let copy s =
  let t = empty () in
    iter s (fun x -> add t x); t

let forall s p =
  fold (fun x a -> if p x then a else false) s true

let exists s p =
  fold (fun x a -> if p x then true else a) s false

let size s = fold (fun x a -> a + 1) s 0

let is_empty s =
  try fold (fun _ _ -> raise Not_found) s true
  with Not_found -> false

let subset s t = 
  forall s (mem t)

let cmp s t =
  subset s t && subset t s

let toList s = fold (fun a b -> a :: b) s []
